==> astk-serveur-1.7.1.1.tbz: aster/ASTK/ASTK_SERV/lib/as_astout.py <==
# -*- coding: iso-8859-1 -*-

# ==============================================================================
# COPYRIGHT (C) 1991 - 2003  EDF R&D                  WWW.CODE-ASTER.ORG
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
# IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
# THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
# (AT YOUR OPTION) ANY LATER VERSION.
#
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
# WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.
#
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
# ==============================================================================

"""
Defines a function to run a list of testcases
"""

__revision__ = "$Id: as_astout.py 3684 2008-12-15 08:01:04Z courtois $"


# ----- differ messages translation
def _(mesg):
   return mesg

import os
import sys
import time
import random

from as_utils       import get_list
from as_thread      import Dispatcher
from as_distrib     import DistribTestTask
from as_common_func import get_hostrc, get_timeout

fmt_res0 = 'testcase     result                cpu      sys    cpu+sys  elapsed'
fmt_resu = '%-12s %-18s %8.2f %8.2f %8.2f %8.2f'
fmt_res2 = '%4d tests   %4d errors        %8.2f %8.2f %8.2f %8.2f'
fmt_tot = '-'*12+' '+'-'*18+' '+'-'*8+' '+'-'*8+' '+'-'*8+' '+'-'*8

#-------------------------------------------------------------------------------
def RunAstout(run, conf, prof, mpi_info, numthread='auto', **kargs):
   """Run a list of test cases...
   """
   run.print_timer = True
   run.timer.Start(_('Tests execution'), hide=True)

   prev = os.getcwd()

   # 1. ----- initializations
   jn = run['num_job']
   REPREF  = os.path.join(run.aster_root, prof['version'][0])

   # 1.2. rep_trav from profile or from run[...]
   reptrav = mpi_info.reptrav()
   
   run.Mess(_('Code_Aster tests execution'), 'TITLE')
   mpi_info.set_cpuinfo(1, 1)
   
   # ----- how many threads ?
   if type(numthread) not in (int, long):
      numthread = run.GetCpuInfo('numthread')

   # 1.3. content of the profile
   ltest = os.path.join(reptrav, 'tests.list')
   if not prof.Get('D', typ='list'):
      run.Mess(_('no list of tests found'), _('<E>_NO_TEST_LIST'))
   else:
      for dli in prof.Get('D', typ='list'):
         if run.IsRemote(dli['path']):
            tmplist = os.path.join(run['tmp_user'], 'list.'+jn)
            run.ToDelete(tmplist)
            kret = run.Copy(tmplist, dli['path'])
            run.FileCat(tmplist, ltest)
         else:
            tmplist = run.PathOnly(dli['path'])
            run.FileCat(tmplist, ltest)
   if not prof.Get('D', typ='rep_test'):
      reptest = []
   else:
      reptest = [r['path'] for r in prof.Get('D', typ='rep_test')]
   if not prof.Get('R', typ='resu_test'):
      run.Mess(_('no result directory found'), ('<E>_NO_RESU_DIR'))
   else:
      resutest = prof.Get('R', typ='resu_test')[0]['path']
      run.MkDir(resutest)
      run.MkDir(os.path.join(resutest, 'flash'))

   facmtps = 1.
   nbmaxnook = 5
   cpresok = 'RESNOOK'
   try:
      if prof['facmtps'][0] != '':
         facmtps   = float(prof['facmtps'][0])
   except ValueError:
      run.Mess(_('incorrect value for %s : %s') % ('facmtps', prof['facmtps'][0]))
   try:
      if prof['nbmaxnook'][0] != '':
         nbmaxnook = int(prof['nbmaxnook'][0])
   except ValueError:
      run.Mess(_('incorrect value for %s : %s') % ('nbmaxnook', prof['nbmaxnook'][0]))
   if prof['cpresok'][0] != '':
      cpresok   = prof['cpresok'][0]
   run.CheckOK()
   
   # get the list of the tests to execute
   iret, list_tests = get_list(ltest, unique=True)
   if iret != 0:
      run.Mess(_('error during reading file : %s') % ltest, _('<F>_ERROR'))
   random.shuffle(list_tests)
   nbtest = len(list_tests)
   
   # suivi des calculs partagé entre 'numthread' threads
   hostrc = get_hostrc(run, prof)
   sum_cpu = hostrc.get_sum('cpu')
   
   # timeout before rejected a job
   timeout = get_timeout(run, prof, 24.*3600.)
   
   # print a summary
   summary = _("""
--- %d test-cases to run
--- Run started at %s
--- Parameters used for this run :
    Directory of reference files    : %s
    Directory of developper files   : %s
    Directory to copy results       : %s
    Code_Aster version              : %s
    Executable filename             : %s
    Commands catalogue directory    : %s
    Elements catalogue filename     : %s
    Maximum number of errors (NOOK) : %d
    Criteria to copy result files   : %s
    Time multiplicative factor      : %f
    Working directory               : %s
    Submission timeout (seconds)    : %.0f""")
    
   sum_thread = _("""    Number of threads               : %d""")
   sum_rc     = _("""    Available hosts (number of cpu) : %s""")
   sum_end = _("""
--- All tests finished at %s

--- Results :
""")
   fmt_spup = _("""
--- Speed-up is %.2f
""")
   info_start = ( nbtest, time.strftime('%c'),
      os.path.join(REPREF, conf['SRCTEST'][0]), ', '.join(reptest), resutest,
      prof['version'][0], kargs['exe'], kargs['cmde'], kargs['ele'],
      nbmaxnook, cpresok, facmtps, reptrav, timeout )
   txt_summary = [ summary % info_start, ]
   if numthread > 1:
      txt_summary.append( sum_thread % numthread )
   if hostrc:
      host_str = ', '.join(['%(host)s (%(cpu)d)' % hostrc.GetConfig(h) for h in hostrc.all_hosts ])
      txt_summary.append( sum_rc % host_str )
   txt_summary.append('')
   text_summary = os.linesep.join(txt_summary)
   print text_summary
   
   # change directory before running sub-execution (reptrav will be deleted !)
   os.chdir(run['rep_trav'])
   
   # ----- Execute tests in parallel using a Dispatcher object
   # elementary task...
   task = DistribTestTask(run=run, prof=prof, conf=conf,  # IN
                          hostrc=hostrc,
                          nbmaxitem=0, timeout=timeout,
                          REPREF=REPREF, reptest=reptest, resutest=resutest,
                          nbmaxnook=nbmaxnook, cpresok=cpresok, facmtps=facmtps,
                          #reptrav=reptrav,
                          info=1,
                          nbnook=0, test_result=[])            # OUT
   # ... and dispatch task on 'list_tests'
   couples = zip(list_tests, [None]*nbtest)
   astout = Dispatcher(couples, task, numthread=numthread)
   
   # Summary
   tit = _('Summary of the run')
   run.Mess(tit, 'TITLE')
   print text_summary
   print sum_end % time.strftime('%c')
   t = [0., 0., 0., 0.]
   print fmt_res0
   print fmt_tot
   task.test_result.sort()
   for result in task.test_result:
      lin = result[:]
      del lin[1]
      t[0] += lin[2]
      t[1] += lin[3]
      t[2] += lin[4]
      t[3] += lin[5]
      print fmt_resu % tuple(lin)
   print fmt_tot
   print fmt_res2 % (len(task.test_result), task.nbnook, t[0], t[1], t[2], t[3])
   
   # check the number of tests really run
   if len(task.test_result) != len(list_tests):
      run.Mess(_('%d test-cases to run, %d really run !') \
         % (len(list_tests), len(task.test_result)), _('<E>_ERROR'))
   
   # force global diagnostic to <F>_ERROR if errors occured
   if task.nbnook > 0:
      run.diag = _('<E>_ERROR')

   cpu_dt, sys_dt, tot_dt = run.timer.StopAndGet(_('Tests execution'))
   spup = 0.
   if tot_dt != 0.:
      spup = t[3]/tot_dt
   print fmt_spup % spup
   if hostrc is not None:
      print os.linesep + hostrc.repr_history()

   run.CheckOK()
   run.Mess(_('All tests run successfully'), 'OK')
   os.chdir(prev)

==> astk-serveur-1.7.4.1.tbz: aster/ASTK/ASTK_SERV/lib/as_astout.py <==
# -*- coding: iso-8859-1 -*-

# ==============================================================================
# COPYRIGHT (C) 1991 - 2003  EDF R&D                  WWW.CODE-ASTER.ORG
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
# IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
# THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
# (AT YOUR OPTION) ANY LATER VERSION.
#
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
# WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.
#
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
# ==============================================================================

"""
Defines a function to run a list of testcases
"""

__revision__ = "$Id: as_astout.py 3876 2009-05-19 16:30:45Z courtois $"


# ----- differ messages translation
def _(mesg):
   return mesg

import os
import sys
import time
import random
import cPickle

from as_utils       import get_list
from as_build       import ASTER_BUILD
from as_thread      import Dispatcher
from as_distrib     import DistribTestTask
from as_common_func import get_hostrc, get_timeout
from as_maintenance import get_version, repr_vers, get_diagnostic

fmt_res0 = 'testcase     result                cpu      sys    cpu+sys  elapsed'
fmt_resu = '%-12s %-18s %8.2f %8.2f %8.2f %8.2f'
fmt_res2 = '%4d tests   %4d errors        %8.2f %8.2f %8.2f %8.2f'
fmt_tot = '-'*12+' '+'-'*18+' '+'-'*8+' '+'-'*8+' '+'-'*8+' '+'-'*8

#-------------------------------------------------------------------------------
def RunAstout(run, conf, prof, mpi_info, numthread='auto', **kargs):
   """Run a list of test cases...
   """
   run.print_timer = True
   run.timer.Start(_('Tests execution'), hide=True)

   prev = os.getcwd()

   # 1. ----- initializations
   jn = run['num_job']
   REPREF  = os.path.join(run.aster_root, prof['version'][0])

   # 1.1. get ASTER_BUILD object
   build = ASTER_BUILD(run, conf)

   # 1.2. rep_trav from profile or from run[...]
   reptrav = mpi_info.reptrav()
   
   run.Mess(_('Code_Aster tests execution'), 'TITLE')
   mpi_info.set_cpuinfo(1, 1)
   
   # ----- how many threads ?
   if type(numthread) not in (int, long):
      numthread = run.GetCpuInfo('numthread')

   # 1.3. content of the profile
   ltest = os.path.join(reptrav, 'tests.list')
   if not prof.Get('D', typ='list'):
      run.Mess(_('no list of tests found'), _('<E>_NO_TEST_LIST'))
   else:
      for dli in prof.Get('D', typ='list'):
         if run.IsRemote(dli['path']):
            tmplist = os.path.join(run['tmp_user'], 'list.'+jn)
            run.ToDelete(tmplist)
            kret = run.Copy(tmplist, dli['path'])
            run.FileCat(tmplist, ltest)
         else:
            tmplist = run.PathOnly(dli['path'])
            run.FileCat(tmplist, ltest)
   if not prof.Get('D', typ='rep_test'):
      reptest = []
   else:
      reptest = [r['path'] for r in prof.Get('D', typ='rep_test')]
   if not prof.Get('R', typ='resu_test'):
      run.Mess(_('no result directory found'), ('<E>_NO_RESU_DIR'))
   else:
      resutest = prof.Get('R', typ='resu_test')[0]['path']
      run.MkDir(resutest)
      flashdir = os.path.join(resutest, 'flash')
      if prof['flashdir'][0] == 'None':
         flashdir = None
      elif prof['flashdir'][0] != '':
         flashdir = prof['flashdir'][0]

   facmtps = 1.
   nbmaxnook = 5
   cpresok = 'RESNOOK'
   try:
      if prof['facmtps'][0] != '':
         facmtps   = float(prof['facmtps'][0])
   except ValueError:
      run.Mess(_('incorrect value for %s : %s') % ('facmtps', prof['facmtps'][0]))
   try:
      if prof['nbmaxnook'][0] != '':
         nbmaxnook = int(prof['nbmaxnook'][0])
   except ValueError:
      run.Mess(_('incorrect value for %s : %s') % ('nbmaxnook', prof['nbmaxnook'][0]))
   if prof['cpresok'][0] != '':
      cpresok   = prof['cpresok'][0]
   run.CheckOK()
   
   # get the list of the tests to execute
   iret, list_tests = get_list(ltest, unique=True)
   if iret != 0:
      run.Mess(_('error during reading file : %s') % ltest, _('<F>_ERROR'))
   nbtest = len(list_tests)
   
   # should we run only nook tests ?
   if run['only_nook']:
      l_dirs = [os.path.join(REPREF, conf['SRCTEST'][0])] + reptest + [resutest]
      old_run_result = get_diagnostic(run, build, l_dirs, list_tests)
      run.DBG('get_diagnostic result', old_run_result, all=True)
      nbini = len(list_tests)
      list_tests = []
      for test, dres in old_run_result.items():
         if test.startswith('__'):
            continue
         if run.GetGrav(dres['diag']) >= run.GetGrav('NOOK'):
            list_tests.append(test)
      nbtest = len(list_tests)
      print _("""
--- %d test-cases to run initially
    %d test-cases previously failed (or their results have not been found) and will be run again
""") % (nbini, nbtest)
   
   random.shuffle(list_tests)

   # suivi des calculs partagé entre 'numthread' threads
   hostrc = get_hostrc(run, prof)
   sum_cpu = hostrc.get_sum('cpu')
   
   # timeout before rejected a job
   timeout = get_timeout(run, prof, 24.*3600.)
   
   # print a summary
   summary = _("""
--- %d test-cases to run
--- Run started at %s
--- Parameters used for this run :
    Directory of reference files    : %s
    Directory of developper files   : %s
    Directory to copy results       : %s
    Directory for job files         : %s
    Code_Aster version              : %s (%s)
    Executable filename             : %s
    Commands catalogue directory    : %s
    Elements catalogue filename     : %s
    Maximum number of errors (NOOK) : %d
    Criteria to copy result files   : %s
    Time multiplicative factor      : %f
    Working directory               : %s
    Submission timeout (seconds)    : %.0f""")
    
   sum_thread = _("""    Number of threads               : %d""")
   sum_rc     = _("""    Available hosts (number of cpu) : %s""")
   sum_end = _("""
--- All tests finished at %s

--- Results :
""")
   fmt_spup = _("""
--- Speed-up is %.2f
""")
   version_number = '.'.join(get_version(run, REPREF)[:3])
   info_start = ( nbtest, time.strftime('%c'),
      os.path.join(REPREF, conf['SRCTEST'][0]), ', '.join(reptest), resutest, flashdir,
      prof['version'][0], version_number, kargs['exe'], kargs['cmde'], kargs['ele'],
      nbmaxnook, cpresok, facmtps, reptrav, timeout )
   txt_summary = [ summary % info_start, ]
   if numthread > 1:
      txt_summary.append( sum_thread % numthread )
   if hostrc:
      host_str = ', '.join(['%(host)s (%(cpu)d)' % hostrc.GetConfig(h) for h in hostrc.all_hosts ])
      txt_summary.append( sum_rc % host_str )
   txt_summary.append('')
   text_summary = os.linesep.join(txt_summary)
   print text_summary
   
   # change directory before running sub-execution (reptrav will be deleted !)
   os.chdir(run['rep_trav'])
   
   # ----- Execute tests in parallel using a Dispatcher object
   # elementary task...
   task = DistribTestTask(run=run, prof=prof, conf=conf,  # IN
                          hostrc=hostrc,
                          nbmaxitem=0, timeout=timeout,
                          REPREF=REPREF, reptest=reptest, resutest=resutest, flashdir=flashdir,
                          nbmaxnook=nbmaxnook, cpresok=cpresok, facmtps=facmtps,
                          #reptrav=reptrav,
                          info=1,
                          nbnook=0, test_result=[])            # OUT
   # ... and dispatch task on 'list_tests'
   couples = zip(list_tests, [None]*nbtest)
   astout = Dispatcher(couples, task, numthread=numthread)
   
   # Summary
   tit = _('Summary of the run')
   run.Mess(tit, 'TITLE')
   print text_summary
   print sum_end % time.strftime('%c')
   t = [0., 0., 0., 0.]
   print fmt_res0
   print fmt_tot
   dict_resu = {
      '__global__' : {
         'astest_dir' : reptest,
         's_astest_dir' : ', '.join(reptest),
         'nbtest'     : len(task.test_result),
         'err_all'    : task.nbnook,
         'err_noresu' : 0,
         'err_vers'   : 0,
         'version'    : version_number,
      }
   }
   task.test_result.sort()
   for result in task.test_result:
      lin = result[:]
      del lin[1]
      t[0] += lin[2]
      t[1] += lin[3]
      t[2] += lin[4]
      t[3] += lin[5]
      print fmt_resu % tuple(lin)
      dict_resu[lin[0]] = {
         'test'  : lin[0],
         'diag'  : lin[1],
         'tcpu'  : lin[2],
         'tsys'  : lin[3],
         'ttot'  : lin[4],
         'vers'  : repr_vers(version_number),
      }
   print fmt_tot
   print fmt_res2 % (len(task.test_result), task.nbnook, t[0], t[1], t[2], t[3])
   
   # write diag.pick
   if prof['diag_pickled'][0] != '':
      pick = open(prof['diag_pickled'][0], 'w')
      cPickle.dump(dict_resu, pick)
   
   # check the number of tests really run
   if len(task.test_result) != len(list_tests):
      run.Mess(_('%d test-cases to run, %d really run !') \
         % (len(list_tests), len(task.test_result)), _('<E>_ERROR'))
   
   # force global diagnostic to <F>_ERROR if errors occured
   if task.nbnook > 0:
      run.diag = _('<E>_ERROR')

   cpu_dt, sys_dt, tot_dt = run.timer.StopAndGet(_('Tests execution'))
   spup = 0.
   if tot_dt != 0.:
      spup = t[3]/tot_dt
   print fmt_spup % spup
   if hostrc is not None:
      print os.linesep + hostrc.repr_history()

   run.CheckOK()
   run.Mess(_('All tests run successfully'), 'OK')
   os.chdir(prev)
